package com.hungraim.ltc.account.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.account.service.AccountService;
import com.hungraim.ltc.pojo.entity.account.ChDisabAccounts;
import com.hungraim.ltc.pojo.vo.account.*;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.util.Date;
import java.util.List;

/**
 * 结算管理控制层
 */
@RestController
@RequestMapping("/api.account/account")
@AllArgsConstructor
@Slf4j
public class AccountController {

    private final AccountService accountService;

    /**
     * 添加或修改结算记录
     *
     * @return 返回状态
     */
    @PostMapping("/addOrUpdateAccount")
    public Result addOrUpdateAccount(AccountOperVo accountOperVo) {
        if (null == accountOperVo.getSrvOrganId() || null == accountOperVo.getAccountsTime() || StringUtils.isBlank(accountOperVo.getTotalExpensesPayable()) || null == accountOperVo.getAccountsStatus()) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        Long accountsId = accountOperVo.getAccountsId();
        Date accountsTime = accountOperVo.getAccountsTime();
        Long srvOrganId = accountOperVo.getSrvOrganId();
        if (null == accountsId) {
            List<ChDisabAccounts> list = accountService.list(new LambdaQueryWrapper<ChDisabAccounts>().eq(ChDisabAccounts::getAccountsTime, accountsTime).eq(ChDisabAccounts::getSrvOrganId, srvOrganId));
            if (null != list && list.size() > 0) {
                return Result.failed("当前机构当月已存在结算数据");
            }
        }
        accountService.addOrUpdateAccount(accountOperVo);
        return Result.success();
    }

    /**
     * 查询结算信息列表
     *
     * @param srvOrganListReq 请求参数对象
     * @return 返回雷彪
     */
    @GetMapping("/searchSrvOrganList")
    public Result<Page<SrvOrganListResp>> searchSrvOrganList(SrvOrganListReq srvOrganListReq) {
        return Result.success(accountService.searchSrvOrganList(srvOrganListReq));
    }

    /**
     * 获取结算详情
     *
     * @param accountsId 结算id
     * @return 返回机构结算详情
     */
    @GetMapping("/srvOrganDetail")
    public Result<AccountOperVo> getSrvOrganDetail(Long accountsId) {
        if (null == accountsId) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        return Result.success(accountService.getSrvOrganDetail(accountsId));
    }

    /**
     * 费用结算
     *
     * @return 返回机构结当月算详情
     */
    @GetMapping("/expenseSettlement")
    public Result<ExpenseVo> expenseSettlement(ExpenseSettlementReq req) throws ParseException {
        if (null == req.getSrvOrganId() || StringUtils.isBlank(req.getAccountsTime())) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (req.getPageNum() == null) {
            req.setPageNum(1);
        }
        if (req.getPageSize() == null) {
            req.setPageSize(10);
        }
        return Result.success(accountService.expenseSettlement(req));
    }
}
