package com.hungraim.ltc.account.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hungraim.ltc.account.dao.*;
import com.hungraim.ltc.account.service.AccountService;
import com.hungraim.ltc.account.service.BaseAccountService;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChCfgCareService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChCfgCostStandard;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.account.ChDisabAccounts;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetail;
import com.hungraim.ltc.pojo.vo.account.*;
import com.hungraim.ltc.util.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

@Service
@AllArgsConstructor
@Slf4j
public class AccountServiceImpl extends ServiceImpl<ChDisabAccountsMapper, ChDisabAccounts> implements AccountService {

    private final AttachFeignService attachFeignService;
    private final BaseAccountService baseAccountService;
    private final ChDisabAccountsMapper disabAccountsMapper;
    private final SrvTaskMapper srvTaskMapper;
    private final ChSrvTaskDetailMapper taskDetailMapper;
    private final ChSrvTaskFinishItemMapper taskFinishItemMapper;
    private final ChSrvProgramMapper programMapper;
    private final ChSrvProgramSetMapper programSetMapper;
    private final ChCfgCareServiceMapper chCfgCareServiceMapper;
    private final ChCfgCostStandardMapper chCfgCostStandardMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addOrUpdateAccount(AccountOperVo accountOperVo) {
        ChDisabAccounts chDisabAccounts = new ChDisabAccounts();
        BeanUtils.copyProperties(accountOperVo, chDisabAccounts);
        chDisabAccounts.setChooseSettle((short) 0); // 是否结算(0未结算，1已结算)
        this.saveOrUpdate(chDisabAccounts);
        accountOperVo.setAccountsId(chDisabAccounts.getAccountsId());
        baseAccountService.updateAttachs(chDisabAccounts.getAccountsId(), accountOperVo.getFileDataList(), AttachType.SRV_ORGAN_ACCOUNT);
    }

    @Override
    public Page<SrvOrganListResp> searchSrvOrganList(SrvOrganListReq srvOrganListReq) {
        List<Long> organIds = baseAccountService.getOrganIds(srvOrganListReq.getOrganId(), srvOrganListReq.getIsInvolveChild());
        return disabAccountsMapper.searchSrvOrganList(new Page<>(srvOrganListReq.getPageNum(), srvOrganListReq.getPageSize()), organIds, srvOrganListReq);
    }

    @Override
    public AccountOperVo getSrvOrganDetail(Long accountsId) {
        AccountOperVo accountOperVo = disabAccountsMapper.getDetailById(accountsId);
        try {
            Result<List<ChFndAttach>> result = attachFeignService.selectAttachs(accountOperVo.getAccountsId(), AttachType.SRV_ORGAN_ACCOUNT.getUploadType());
            if (ResultCode.SUCCESS.getCode().equals(result.getCode())) {
                accountOperVo.setFileDataList(result.getData());
            } else {
                log.error("获取机构结算资料失败：{}", result.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取机构结算资料失败：{}", e.getMessage());
        }
        return accountOperVo;
    }

    @Override
    public ExpenseVo expenseSettlement(ExpenseSettlementReq req) throws ParseException {
        Page<ExpenseSettlementInfoBo> page = new Page<>(req.getPageNum(), req.getPageSize());
        //计算当前月最后一天
        DateFormat df = new SimpleDateFormat("yyyy-MM");
        Date parse = df.parse(req.getAccountsTime());
        Date lastDayOfMonth = getLastDayOfMonth(parse);
        DateFormat df2 = new SimpleDateFormat("yyyy-MM-dd");
        req.setAccountsLastDayTime(df2.format(lastDayOfMonth));
        Page<ExpenseSettlementInfoBo> expenseSettlementInfo = srvTaskMapper.getExpenseSettlementInfo(page, req);
        List<ExpenseSettlementVo> vos = new ArrayList<>();
        if (expenseSettlementInfo != null && expenseSettlementInfo.getRecords() != null) {
            for (ExpenseSettlementInfoBo bo : expenseSettlementInfo.getRecords()) {
                ExpenseSettlementVo vo = new ExpenseSettlementVo();
                BeanUtil.copyProperties(bo, vo);
                //护理等级
                vo.setNursingGrade(CfgSrvLevel.getMsg(bo.getSrvLevelId()));
                //服务开始时间
                if (bo.getSrvStartTime() != null) {
                    if (bo.getSrvStartTime().getTime() > parse.getTime()) {
                        vo.setSrvStartTime(bo.getSrvStartTime());
                    } else {
                        vo.setSrvStartTime(parse);
                    }
                }
                //服务结束时间
                if (bo.getSrvEndTime() != null) {
                    if (bo.getSrvEndTime().getTime() > lastDayOfMonth.getTime()) {
                        vo.setSrvEndTime(lastDayOfMonth);
                    } else {
                        vo.setSrvEndTime(bo.getSrvEndTime());
                    }
                }
                //有效结算日
                int startDay = getDay(vo.getSrvStartTime());
                int endDay = getDay(vo.getSrvEndTime());
                vo.setEffectiveSettlementDate(endDay - startDay + 1);
                //基护费用标准
                Integer effectiveSettlementDate = vo.getEffectiveSettlementDate();//有效结算日
                ChCfgCostStandard costStandard = chCfgCostStandardMapper.selectOne(new LambdaQueryWrapper<ChCfgCostStandard>()
                        .eq(ChCfgCostStandard::getOrganId, bo.getOrganId())
                        .eq(ChCfgCostStandard::getSrvLevelId, bo.getSrvLevelId())
                        .eq(ChCfgCostStandard::getSrvModeId, bo.getSrvModeId()));//基护费用标准(可配置)
                vo.setCostStandard(costStandard + "元/人/天");
                //基护费用
                BigDecimal foundation = BigDecimal.valueOf(costStandard.getCost()).multiply(BigDecimal.valueOf(effectiveSettlementDate));
                vo.setFoundationProtectionCost(foundation.doubleValue());
                //基护完成率
                Integer srvTaskId = bo.getSrvTaskId();
                int taskDetailsCount = taskDetailMapper.selectCount(new LambdaQueryWrapper<ChSrvTaskDetail>()//计划次数
                        .eq(ChSrvTaskDetail::getSrvTaskId, srvTaskId)
                        .in(ChSrvTaskDetail::getIllegalType, 0, 1)
                        .ne(ChSrvTaskDetail::getStatus, 3));
                List<ChSrvTaskDetail> taskDetails = taskDetailMapper.selectList(new LambdaQueryWrapper<ChSrvTaskDetail>()
                        .eq(ChSrvTaskDetail::getSrvTaskId, srvTaskId)
                        .in(ChSrvTaskDetail::getIllegalType, 0, 1)
                        .eq(ChSrvTaskDetail::getStatus, 4));
                int taskDetailsFinishCount = 0;//完成次数
                if (taskDetails != null && taskDetails.size() > 0) {
                    taskDetailsFinishCount = taskDetails.size();
                }
                int monPlanningFrequency = bo.getPlanningFrequency();//每月计划次数
                if (bo.getPlanningFrequencyType() == 1) {
                    monPlanningFrequency = bo.getPlanningFrequency();
                } else if (bo.getPlanningFrequencyType() == 2) {
                    monPlanningFrequency = bo.getPlanningFrequency() * 30;
                } else if (bo.getPlanningFrequencyType() == 3) {
                    monPlanningFrequency = bo.getPlanningFrequency() * 4;
                }
                BigDecimal divide;
                if (taskDetailsCount == 0) {
                    divide = BigDecimal.valueOf(0);
                } else {
                    divide = BigDecimal.valueOf(taskDetailsFinishCount).divide(BigDecimal.valueOf(taskDetailsCount), 2, BigDecimal.ROUND_DOWN);
                }
                BigDecimal divide1 = BigDecimal.valueOf(bo.getItemTimes()).divide(BigDecimal.valueOf(monPlanningFrequency), 2, BigDecimal.ROUND_DOWN);
                if (divide.doubleValue() > 1) {
                    divide = BigDecimal.valueOf(1);
                }
                if (divide1.doubleValue() > 1) {
                    divide1 = BigDecimal.valueOf(1);
                }
                BigDecimal comp = divide.doubleValue() == 0 ? BigDecimal.valueOf(0) : divide.multiply(divide1).setScale(4, BigDecimal.ROUND_DOWN);//基护完成率
                vo.setCompletionRateFoundationProtection(comp.doubleValue() == 0 ? "0%" : comp.multiply(BigDecimal.valueOf(100)).doubleValue() + "%");
                //巡护基护完成考核
                vo.setPatrolAndBasicProtectionCompletedAssessment("-");
                BigDecimal patrolAndBasicProtectionCompletedAssessment = BigDecimal.valueOf(0.8);//巡护基护完成考核
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    long serviceTime = 0;//服务时长/毫秒
                    int serviceCount = 0;//服务次数
                    if (taskDetails != null && taskDetails.size() > 0) {
                        for (ChSrvTaskDetail taskDetail : taskDetails) {
                            long l = taskDetail.getSrvEndTime().getTime() - taskDetail.getSrvStartTime().getTime();
                            serviceTime += l;
                            if (l > 30 * 60 * 1000) {
                                serviceCount++;
                            }
                        }
                    }
                    long serviceTimeM = 0;
                    if (serviceTime != 0) {
                        serviceTimeM = serviceTime / (1000 * 60 * 60);//服务时长/小时
                    }
                    boolean serviceCountK = false;//服务次数考核
                    boolean serviceTimeK = false;//服务时长考核
                    if (serviceCount >= 6) {
                        serviceCountK = true;
                    }
                    if (DisableLevel.ONE.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 8) {
                            serviceTimeK = true;
                        }
                    } else if (DisableLevel.TWO.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 10) {
                            serviceTimeK = true;
                        }
                    } else if (DisableLevel.THREE.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 12) {
                            serviceTimeK = true;
                        }
                    }
                    if (serviceCountK && serviceTimeK) {
                        patrolAndBasicProtectionCompletedAssessment = BigDecimal.valueOf(1);
                        vo.setPatrolAndBasicProtectionCompletedAssessment("100%");
                    } else if (serviceCountK || serviceTimeK) {
                        patrolAndBasicProtectionCompletedAssessment = BigDecimal.valueOf(0.9);
                        vo.setPatrolAndBasicProtectionCompletedAssessment("90%");
                    } else {
                        vo.setPatrolAndBasicProtectionCompletedAssessment("80%");
                    }
                }
                //基护应拨费用
                BigDecimal foundationProtectionCost = BigDecimal.valueOf(vo.getFoundationProtectionCost());//基护费用
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    BigDecimal multiply = foundationProtectionCost.multiply(comp).multiply(patrolAndBasicProtectionCompletedAssessment).multiply(BigDecimal.valueOf(0.9)).setScale(2, BigDecimal.ROUND_DOWN);
                    vo.setExpensesPayableFoundationProtection(multiply.doubleValue());
                } else if (FndSrvMode.HOME_CARE.getCode().equals(bo.getSrvModeId())
                        || FndSrvMode.HOSPITAL_NURSING.getCode().equals(bo.getSrvModeId())) {//家护/院护
                    BigDecimal multiply = foundationProtectionCost.multiply(comp).multiply(BigDecimal.valueOf(0.9)).setScale(2, BigDecimal.ROUND_DOWN);
                    vo.setExpensesPayableFoundationProtection(multiply.doubleValue());
                }
                //照护费用标准
                int standardCareCost = 0;
                if (CfgSrvLevel.ONE.getCode().equals(bo.getSrvLevelId())) {//服务等级
                    standardCareCost = 25;
                    vo.setStandardCareCost("25元/人/天");
                } else if (CfgSrvLevel.TWO.getCode().equals(bo.getSrvLevelId())) {
                    standardCareCost = 35;
                    vo.setStandardCareCost("35元/人/天");
                } else if (CfgSrvLevel.THREE.getCode().equals(bo.getSrvLevelId())) {
                    standardCareCost = 45;
                    vo.setStandardCareCost("45元/人/天");
                }
                //照护费用
                List<ChSrvTaskDetail> taskDetails2 = taskDetailMapper.selectList(new LambdaQueryWrapper<ChSrvTaskDetail>()
                        .eq(ChSrvTaskDetail::getSrvTaskId, srvTaskId)
                        .in(ChSrvTaskDetail::getIllegalType, 0, 2)
                        .eq(ChSrvTaskDetail::getStatus, 4));
                BigDecimal careCosts = BigDecimal.valueOf(0);
                int taskDetailsFinishCount2 = 0;
                if (taskDetails2 != null && taskDetails2.size() > 0) {
                    taskDetailsFinishCount2 = taskDetails2.size();
                    careCosts = BigDecimal.valueOf(standardCareCost).multiply(BigDecimal.valueOf(taskDetails2.size()));
                }
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    ChCfgCareService chCfgCareService = chCfgCareServiceMapper.selectOne(new LambdaQueryWrapper<ChCfgCareService>()
                            .eq(ChCfgCareService::getSrvlevelId, bo.getSrvLevelId()));//根据服务等级查询照护服务费用限额配置表
                    if (chCfgCareService != null) {
                        if (careCosts.intValue() > chCfgCareService.getMonthlyPaymentLimit()) {
                            careCosts = BigDecimal.valueOf(chCfgCareService.getMonthlyPaymentLimit());
                        }
                    }
                }
                vo.setCareCosts(careCosts.doubleValue());
                //照护完成率
                int taskDetailsCount2 = taskDetailMapper.selectCount(new LambdaQueryWrapper<ChSrvTaskDetail>()//计划次数
                        .eq(ChSrvTaskDetail::getSrvTaskId, srvTaskId)
                        .in(ChSrvTaskDetail::getIllegalType, 0, 2)
                        .ne(ChSrvTaskDetail::getStatus, 3));
                BigDecimal divide2;
                if (taskDetailsCount2 == 0) {
                    divide2 = BigDecimal.valueOf(0);
                } else {
                    divide2 = BigDecimal.valueOf(taskDetailsFinishCount2).divide(BigDecimal.valueOf(taskDetailsCount2), 2, BigDecimal.ROUND_DOWN);
                }
                BigDecimal divide3 = BigDecimal.valueOf(bo.getItemTimes()).divide(BigDecimal.valueOf(monPlanningFrequency), 2, BigDecimal.ROUND_DOWN);
                if (divide2.doubleValue() > 1) {
                    divide2 = BigDecimal.valueOf(1);
                }
                if (divide3.doubleValue() > 1) {
                    divide3 = BigDecimal.valueOf(1);
                }
                BigDecimal comp2 = divide2.doubleValue() == 0 ? BigDecimal.valueOf(0) : divide2.multiply(divide3).setScale(4, BigDecimal.ROUND_DOWN);//照护完成率
                vo.setCareCompletionRate(comp2.doubleValue() == 0 ? "0%" : comp2.multiply(BigDecimal.valueOf(100)).doubleValue() + "%");
                //巡护照护完成考核
                vo.setPatrolCareCompletionAssessment("-");
                BigDecimal patrolCareCompletionAssessment = BigDecimal.valueOf(0.8);//巡护照护完成考核
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    long serviceTime = 0;//服务时长/毫秒
                    int serviceCount = 0;//服务次数
                    if (taskDetails2 != null && taskDetails2.size() > 0) {
                        for (ChSrvTaskDetail taskDetail : taskDetails2) {
                            long l = taskDetail.getSrvEndTime().getTime() - taskDetail.getSrvStartTime().getTime();
                            serviceTime += l;
                            if (l > 30 * 60 * 1000) {
                                serviceCount++;
                            }
                        }
                    }
                    long serviceTimeM = 0;
                    if (serviceTime != 0) {
                        serviceTimeM = serviceTime / (1000 * 60 * 60);//服务时长/小时
                    }
                    boolean serviceCountK = false;//服务次数考核
                    boolean serviceTimeK = false;//服务时长考核
                    if (serviceCount >= 6) {
                        serviceCountK = true;
                    }
                    if (DisableLevel.ONE.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 8) {
                            serviceTimeK = true;
                        }
                    } else if (DisableLevel.TWO.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 10) {
                            serviceTimeK = true;
                        }
                    } else if (DisableLevel.THREE.getCode().equals(bo.getDisableLevelId())) {
                        if (serviceTimeM >= 12) {
                            serviceTimeK = true;
                        }
                    }
                    if (serviceCountK && serviceTimeK) {
                        patrolCareCompletionAssessment = BigDecimal.valueOf(1);
                        vo.setPatrolCareCompletionAssessment("100%");
                    } else if (serviceCountK || serviceTimeK) {
                        patrolCareCompletionAssessment = BigDecimal.valueOf(0.9);
                        vo.setPatrolCareCompletionAssessment("90%");
                    } else {
                        vo.setPatrolCareCompletionAssessment("80%");
                    }
                }
                //照护应拨费用
                if (FndSrvMode.PATROL.getCode().equals(bo.getSrvModeId())) {//巡护
                    BigDecimal multiply = careCosts.multiply(comp2).multiply(patrolCareCompletionAssessment).multiply(BigDecimal.valueOf(0.9)).setScale(2, BigDecimal.ROUND_DOWN);
                    vo.setCarePayable(multiply.doubleValue());
                } else if (FndSrvMode.HOME_CARE.getCode().equals(bo.getSrvModeId())
                        || FndSrvMode.HOSPITAL_NURSING.getCode().equals(bo.getSrvModeId())) {//家护/院护
                    BigDecimal multiply = careCosts.multiply(comp2).multiply(BigDecimal.valueOf(0.9)).setScale(2, BigDecimal.ROUND_DOWN);
                    vo.setCarePayable(multiply.doubleValue());
                }
                //应拨付总金额
                vo.setTotalAmountDue(BigDecimal.valueOf(vo.getExpensesPayableFoundationProtection()).add(BigDecimal.valueOf(vo.getCarePayable())).doubleValue());
                vos.add(vo);
            }
            //合计
            ExpenseSettlementTotalVo totalVo = new ExpenseSettlementTotalVo();
            BigDecimal foundationProtectionCost = BigDecimal.valueOf(0);
            BigDecimal expensesPayableFoundationProtection = BigDecimal.valueOf(0);
            BigDecimal careCosts = BigDecimal.valueOf(0);
            BigDecimal carePayable = BigDecimal.valueOf(0);
            BigDecimal totalAmountDue = BigDecimal.valueOf(0);
            for (ExpenseSettlementVo vo : vos) {
                //基护费用
                foundationProtectionCost = foundationProtectionCost.add(BigDecimal.valueOf(vo.getFoundationProtectionCost()));
                //基护应拨费用
                expensesPayableFoundationProtection = expensesPayableFoundationProtection.add(BigDecimal.valueOf(vo.getExpensesPayableFoundationProtection()));
                //照护费用
                careCosts = careCosts.add(BigDecimal.valueOf(vo.getCareCosts()));
                //照护应拨费用
                carePayable = carePayable.add(BigDecimal.valueOf(vo.getCarePayable()));
                //应拨付总金额
                totalAmountDue = totalAmountDue.add(BigDecimal.valueOf(vo.getTotalAmountDue()));
            }
            totalVo.setFoundationProtectionCost(foundationProtectionCost.doubleValue());
            totalVo.setExpensesPayableFoundationProtection(expensesPayableFoundationProtection.doubleValue());
            totalVo.setCareCosts(careCosts.doubleValue());
            totalVo.setCarePayable(carePayable.doubleValue());
            totalVo.setTotalAmountDue(totalAmountDue.doubleValue());
            ExpenseVo expenseVo = new ExpenseVo();
            expenseVo.setTotal(expenseSettlementInfo.getTotal());
            expenseVo.setCurrent(expenseSettlementInfo.getCurrent());
            expenseVo.setSize(expenseSettlementInfo.getSize());
            expenseVo.setExpenseSettlements(vos);
            expenseVo.setTotalVo(totalVo);
            return expenseVo;
        }
        return new ExpenseVo();
    }

    //求当月最后一天
    private Date getLastDayOfMonth(Date date) {
        Calendar cale = Calendar.getInstance();
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        cale.setTime(date);
        cale.add(Calendar.MONTH, 1);
        cale.set(Calendar.DAY_OF_MONTH, 0);
        return cale.getTime();
    }

    //求年月日中的日
    private int getDay(Date date) {
        Calendar cale = Calendar.getInstance();
        cale.setTime(date);
        return cale.get(Calendar.DAY_OF_MONTH);
    }

}
